/* FILE: spool.c */

#include <windows.h>
#include <stdio.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <print.h>
#include <commdlg.h>
#include <string.h>
#include <pro_ext.h>

// function prototypes
int PrintFile(LPSTR, LPSTR, HDC, HGLOBAL, HGLOBAL);
VOID SendFile(HDC, LPSTR, LPSTR);
HDC  GetPrinterDC(HGLOBAL, HGLOBAL);
BOOL CALLBACK __export PrintAbortProc(HDC, int);

// play with this number, but don't go over 32k
#define BUFSIZE 1024

// convenient structure for use with PASSTHROUGH escape
typedef struct
{
  WORD wSize;
  BYTE bData[2];
} PASSTHROUGHSTRUCT, FAR *LPPTS;


BOOL bAbort;        // global printing abort flag


//*************************************************************
//
//  PrintFile()
//
//  Purpose:
//		Reads a file and copies it to a printer using the 
//    PASSTHROUGH escape.
//
//  Parameters:
//      LPSTR   szFile    - Pointer to path/filename to print
//      HDC     hPrnDC    - Handle to printer DC or NULL
//      HGLOBAL hDevNames - Handle to DEVNAMES struct or NULL
//      HGLOBAL hDevMode  - Handle to DEVMODE struct or NULL
//      
//  Return:
//      Returns non-zero for success or zero for failure.
//
//  Comments:
//      hDevNames and hDevMode are only used if hPrnDC is NULL.
//      If both hPrnDC and hDevNames are NULL, the default
//      printer is used.
//
//*************************************************************

int PrintFile ( LPSTR   szFile, LPSTR szJobName,
                            HDC     hPrnDC,
                            HGLOBAL hDevNames,
                            HGLOBAL hDevMode )
{
  int iEsc;
  
  BOOL bLocalDC = TRUE; // assume we must create a DC (hPrnDC == NULL)

  bAbort = FALSE;       // haven't aborted yet


  // make sure we have a printer DC
  
  if (!hPrnDC)
    hPrnDC = GetPrinterDC(hDevNames, hDevMode);
  else
    bLocalDC = FALSE;   // use passed in hPrnDC
    
  if (!hPrnDC)
    return FALSE;

  
  // PASSTHROUGH is required.  If driver doesn't support it, bail out.

  iEsc = PASSTHROUGH;
  if (!Escape(hPrnDC, QUERYESCSUPPORT, sizeof(int), (LPSTR)&iEsc, NULL))
    {
    bAbort = TRUE;
    MessageBox(NULL,"PASSTHROUGH Not Supported","Warning",MB_ICONSTOP|MB_OK);
    goto PFCleanUp;
    }


  // If we created the DC, install an abort procedure.  We don't have
  // a cancel dialog, but the abort proc enables multi-tasking.
  // (Use __export and compile with -GA or -GD so we don't need
  // a MakeProcInstance.)

  if (bLocalDC)
    Escape (hPrnDC, SETABORTPROC, 0, (LPSTR) PrintAbortProc, NULL);


  // Call EPSPRINTING if it is supported (i.e. if we're on a 
  // PostScript printer) to suppress downloading the pscript header.

  iEsc = EPSPRINTING;
  if (Escape(hPrnDC, QUERYESCSUPPORT, sizeof(int), (LPSTR)&iEsc, NULL))
    {
    iEsc = 1;  // 1 == enable PASSTHROUGH (disable pscript header)
    Escape(hPrnDC, EPSPRINTING, sizeof(int), (LPSTR)&iEsc, NULL);
    }

  SendFile(hPrnDC, szFile, szJobName); // send file to printer (could do multiple files)


PFCleanUp:                  // done

  if (bLocalDC)             // only delete DC if we created it
    DeleteDC(hPrnDC);

  return !bAbort;
} /* PrintFile() */


VOID SendFile(HDC hPrnDC, LPSTR szFile, LPSTR szJobName)
{
  static LPPTS lpPTS=NULL;          // pointer to PASSTHROUGHSTRUCT
  OFSTRUCT ofs;
  HFILE hFile;
  
  
  hFile = OpenFile((LPSTR) szFile, &ofs, OF_READ);
  if (hFile == HFILE_ERROR)
    {
    bAbort = TRUE;  // can't open file!
    return;
    }
    

  if (!lpPTS && 
       !(lpPTS = (LPPTS)GlobalLock(GlobalAlloc(GPTR, sizeof(WORD) + BUFSIZE))))
    {
    bAbort = TRUE;  // can't allocate memory for buffer!
    return;
    }

  Escape (hPrnDC, STARTDOC, strlen(szJobName), szJobName, NULL);


  // Loop through the file, reading a chunk at a time and passing
  // it to the printer.  QueryAbort calls the abort proc which
  // processes messages so we don't tie up the whole system.
  // We could skip the QueryAbort, in which case we wouldn't need
  // to set an abort proc at all.

  do {
    if ((lpPTS->wSize=_lread(hFile,(LPSTR)lpPTS->bData,BUFSIZE)) == HFILE_ERROR)
      {
      bAbort = TRUE;  // error reading file
      break;
      }

    Escape(hPrnDC, PASSTHROUGH, NULL, (LPSTR)lpPTS, NULL);
    }
  while ((lpPTS->wSize == BUFSIZE) && QueryAbort(hPrnDC, 0));

  if (!bAbort)
    Escape(hPrnDC, ENDDOC, NULL, NULL, NULL);
    
  if (bAbort)
  {
  	Escape(hPrnDC, ENDDOC, NULL, NULL, NULL);
  	Escape(hPrnDC, ABORTDOC, NULL, NULL, NULL);
  }

  _lclose(hFile);
} /* SendFile() */



HDC GetPrinterDC(HGLOBAL hDevNames, HGLOBAL hDevMode)
{
  HDC hdc;
  char szPrinter[64];
  LPSTR szDevice=NULL, szDriver=NULL, szOutput=NULL;
  LPDEVMODE lpdm=NULL;

  if (hDevNames)
    {
    LPDEVNAMES lpdn = (LPDEVNAMES) GlobalLock(hDevNames);

    szDriver = (LPSTR) lpdn + lpdn->wDriverOffset;
    szDevice = (LPSTR) lpdn + lpdn->wDeviceOffset;
    szOutput = (LPSTR) lpdn + lpdn->wOutputOffset;
    }
  else
    {                       // get default printer info
    GetProfileString ("windows", "device", "", szPrinter, 64);

    if (!((szDevice = strtok (szPrinter, "," )) &&
          (szDriver = strtok (NULL,      ", ")) &&
          (szOutput = strtok (NULL,      ", "))))
            return NULL;    // no default printer
    }

  if (hDevMode)
    lpdm = (LPDEVMODE) GlobalLock(hDevMode);

  hdc = CreateDC(szDriver, szDevice, szOutput, lpdm);

  if (hDevMode)
    GlobalUnlock(hDevMode);
  if (hDevNames)
    GlobalUnlock(hDevNames);

  return hdc;
} /* GetPrinterDC() */


BOOL CALLBACK __export PrintAbortProc(HDC hdc, int code)
{
  
  if (_InKey(-1,HIDECURSOR) == 27)
  	bAbort = TRUE;
  	
  return (!bAbort);
}  /* PrintAbortProc() */

/*** EOF: myspool.c ***/
